@@ -7,7 +7,9 @@
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
-#include "afslogon.h"
+#include <afsconfig.h>
+#include <afs/param.h>
+#include <roken.h>
 
 #include <io.h>
 #include <sys/stat.h>
@@ -15,10 +17,15 @@
 #include <fcntl.h>
 
 #include <winsock2.h>
+#include <winioctl.h>
+#define SECURITY_WIN32
+#include <sspi.h>
 #include <lm.h>
 #include <nb30.h>
+#include <sddl.h>
+
+#include "afslogon.h"
 
-#include <afs/param.h>
 #include <afs/stds.h>
 #include <afs/pioctl_nt.h>
 #include <afs/kautils.h>
@@ -50,11 +57,6 @@ void DebugEvent0(char *a)
     if (!Debug && !ISLOGONTRACE(TraceOption))
         return;
 
-    if (Debug & 2) {
-        OutputDebugString(a);
-        OutputDebugString("\r\n");
-    }
-
     h = RegisterEventSource(NULL, AFS_LOGON_EVENT_NAME);
     if (h != INVALID_HANDLE_VALUE) {
         ptbuf[0] = a;
@@ -66,29 +68,22 @@ void DebugEvent0(char *a)
 #define MAXBUF_ 512
 void DebugEvent(char *b,...)
 {
-    HANDLE h;
-    char *ptbuf[1], buf[MAXBUF_+1];
+    HANDLE h; char *ptbuf[1],buf[MAXBUF_+1];
     va_list marker;
 
     if (!Debug && !ISLOGONTRACE(TraceOption))
         return;
 
-    va_start(marker,b);
-    StringCbVPrintf(buf, MAXBUF_+1,b,marker);
-    buf[MAXBUF_] = '\0';
-
-    if (Debug & 2) {
-        OutputDebugString(buf);
-        OutputDebugString("\r\n");
-    }
-
     h = RegisterEventSource(NULL, AFS_LOGON_EVENT_NAME);
     if (h != INVALID_HANDLE_VALUE) {
+        va_start(marker,b);
+        StringCbVPrintf(buf, MAXBUF_+1,b,marker);
+        buf[MAXBUF_] = '\0';
         ptbuf[0] = buf;
         ReportEvent(h, EVENTLOG_INFORMATION_TYPE, 0, 1008, NULL, 1, 0, (const char **)ptbuf, NULL);
         DeregisterEventSource(h);
+        va_end(marker);
     }
-    va_end(marker);
 }
 
 static HANDLE hInitMutex = NULL;
@@ -219,17 +214,14 @@ DWORD MapAuthError(DWORD code)
     switch (code) {
         /* Unfortunately, returning WN_NO_NETWORK results in the MPR abandoning
          * logon scripts for all credential managers, although they will still
-         * receive logon notifications.
-         *
-         * Instead return WN_NET_ERROR (ERROR_UNEXP_NET_ERR) to indicate a
-         * problem with this network.
+         * receive logon notifications.  Since we don't want this, we return
+         * WN_SUCCESS.  This is highly undesirable, but we also don't want to
+         * break other network providers.
          */
-    case KTC_NOCM:
+ /* case KTC_NOCM:
     case KTC_NOCMRPC:
-        return WN_NET_ERROR;
-
-    default:
-        return WN_SUCCESS;
+    return WN_NO_NETWORK; */
+    default: return WN_SUCCESS;
   }
 }
 
@@ -484,13 +476,7 @@ GetDomainLogonOptions( PLUID lpLogonId, char * username, char * domain, LogonOpt
     }
 
     /* come up with SMB username */
-    if(ISHIGHSECURITY(opt->LogonOption)) {
-        DebugEvent0("High Security Mode active");
-        opt->smbName = malloc( MAXRANDOMNAMELEN );
-        if (opt->smbName == NULL)
-            goto cleanup;
-        GenRandomName(opt->smbName);
-    } else if (lpLogonId) {
+    if (lpLogonId) {
         /* username and domain for logon session is not necessarily the same as
            username and domain passed into network provider. */
         PSECURITY_LOGON_SESSION_DATA plsd=NULL;
@@ -576,14 +562,7 @@ GetDomainLogonOptions( PLUID lpLogonId, char * username, char * domain, LogonOpt
             hkTemp = hkDoms;
             DebugEvent0("Located logon script in hkDoms");
         }
-        /* Note that the LogonScript in the NP key is only used if we are doing high security. */
-        else if(hkNp && ISHIGHSECURITY(opt->LogonOption)) {
-            rv = RegQueryValueExW(hkNp, REG_CLIENT_LOGON_SCRIPT_PARMW, 0, &dwType, NULL, &dwSize);
-            if(rv == ERROR_SUCCESS && !hkTemp && (dwType == REG_SZ || dwType == REG_EXPAND_SZ)) {
-                hkTemp = hkNp;
-                DebugEvent0("Located logon script in hkNp");
-            }
-        }
+        /* Note that the LogonScript in the NP key not used. */
     }
 
     if(hkTemp) {
@@ -822,7 +801,8 @@ UnicodeStringToANSI(UNICODE_STRING uInputString, LPSTR lpszOutputString, int nOu
     return FALSE;
 }  // UnicodeStringToANSI
 
-DWORD APIENTRY NPLogonNotify(
+DWORD APIENTRY
+NPLogonNotify(
 	PLUID lpLogonId,
 	LPCWSTR lpAuthentInfoType,
 	LPVOID lpAuthentInfo,
@@ -862,6 +842,8 @@ DWORD APIENTRY NPLogonNotify(
     int retryInterval;
     int sleepInterval;
 
+    CtxtHandle LogonContext;
+
     /* Are we interactive? */
     interactive = (wcsicmp(lpStationName, L"WinSta0") == 0);
 
@@ -964,7 +946,7 @@ DWORD APIENTRY NPLogonNotify(
         /* Get cell name if doing integrated logon.
            We might overwrite this if we are logging into an AD realm and we find out that
            the user's home dir is in some other cell. */
-        DebugEvent("About to call cm_GetRootCellName()");
+        DebugEvent0("About to call cm_GetRootCellName()");
         code = cm_GetRootCellName(cell);
         if (code < 0) {
             DebugEvent0("Unable to obtain Root Cell");
@@ -984,7 +966,8 @@ DWORD APIENTRY NPLogonNotify(
         }
     }
 
-    /* loop until AFS is started. */
+    AFSCreatePAG(lpLogonId);
+
     if (afsWillAutoStart) {
         /*
          * If the service is configured for auto start but hasn't started yet,
@@ -993,12 +976,12 @@ DWORD APIENTRY NPLogonNotify(
         if (!(IsServiceRunning() || IsServiceStartPending()))
             StartTheService();
 
+        /* loop until AFS is started or fails. */
         while ( IsServiceStartPending() ) {
             Sleep(10);
         }
 
         while (IsServiceRunning() && code != KTC_NOCM && code != KTC_NOCMRPC && code != KTC_NOCELL) {
-
             DebugEvent("while(autostart) LogonOption[%x], Service AutoStart[%d]",
 			opt.LogonOption,afsWillAutoStart);
 
@@ -1016,6 +999,9 @@ DWORD APIENTRY NPLogonNotify(
 	    /* if Integrated Logon  */
 	    if (ISLOGONINTEGRATED(opt.LogonOption))
 	    {
+                LogonSSP(lpLogonId, &LogonContext);
+                ImpersonateSecurityContext(&LogonContext);
+
 		if ( KFW_is_available() ) {
 		    SetEnvironmentVariable(DO_NOT_REGISTER_VARNAME, "");
                     if (opt.realm) {
@@ -1089,6 +1075,10 @@ DWORD APIENTRY NPLogonNotify(
 		    DebugEvent("AFS AfsLogon - (INTEGRATED only)ka_UserAuthenticateGeneral2 Code[%x] uname[%s] smbname=[%s] Cell[%s] PwExp=[%d] Reason=[%s]",
 				code,uname,opt.smbName,cell,pw_exp,reason?reason:"");
 		}
+
+                RevertSecurityContext(&LogonContext);
+                DeleteSecurityContext(&LogonContext);
+
 		if ( code && code != KTC_NOCM && code != KTC_NOCMRPC && !lowercased_name ) {
 		    for ( ctemp = uname; *ctemp ; ctemp++) {
 			*ctemp = tolower(*ctemp);
@@ -1140,8 +1130,8 @@ DWORD APIENTRY NPLogonNotify(
 	    Sleep(sleepInterval * 1000);
 	    retryInterval -= sleepInterval;
 	}
+        DebugEvent0("while loop exited");
     }
-    DebugEvent0("while loop exited");
 
     /* remove any kerberos 5 tickets currently held by the SYSTEM account
      * for this user
@@ -1160,12 +1150,10 @@ DWORD APIENTRY NPLogonNotify(
 	HANDLE h;
 	char *ptbuf[1];
 
-        DebugEvent("Integrated login failed: %s", reason);
-
 	StringCbPrintf(msg, sizeof(msg), "Integrated login failed: %s", reason);
 
 	if (ISLOGONINTEGRATED(opt.LogonOption) && interactive && !opt.failSilently)
-	    MessageBox(hwndOwner, msg, "AFS Logon", MB_OK|MB_ICONWARNING|MB_SYSTEMMODAL);
+	    MessageBox(hwndOwner, msg, "AFS Logon", MB_OK);
 
 	h = RegisterEventSource(NULL, AFS_LOGON_EVENT_NAME);
 	ptbuf[0] = msg;
@@ -1190,8 +1178,6 @@ DWORD APIENTRY NPLogonNotify(
     if (opt.smbName) free(opt.smbName);
     if (opt.realm) free(opt.realm);
 
-    SecureZeroMemory(password, sizeof(password));
-
     DebugEvent("AFS AfsLogon - Exit","Return Code[%x]",code);
     return code;
 }
@@ -1269,14 +1255,6 @@ VOID AFS_Startup_Event( PWLX_NOTIFICATION_INFO pInfo )
     RegQueryValueEx(NPKey, REG_CLIENT_TRACE_OPTION_PARM, NULL,
                      &LSPtype, (LPBYTE)&TraceOption, &LSPsize);
 
-    RegCloseKey (NPKey);
-
-    (void) RegOpenKeyEx(HKEY_LOCAL_MACHINE, AFSREG_CLT_SVC_PROVIDER_SUBKEY,
-                         0, KEY_QUERY_VALUE, &NPKey);
-    LSPsize=sizeof(Debug);
-    RegQueryValueEx(NPKey, REG_CLIENT_DEBUG_PARM, NULL,
-                     &LSPtype, (LPBYTE)&Debug, &LSPsize);
-
     RegCloseKey (NPKey);
     DebugEvent0("AFS_Startup_Event");
 }
@@ -1314,11 +1292,11 @@ VOID AFS_Logoff_Event( PWLX_NOTIFICATION_INFO pInfo )
 	    size_t szlen = 0;
 
 	    StringCchLengthW(pInfo->UserName, MAX_USERNAME_LENGTH, &szlen);
-	    WideCharToMultiByte(CP_UTF8, 0, pInfo->UserName, (int)szlen,
+	    WideCharToMultiByte(CP_UTF8, 0, pInfo->UserName, szlen,
 				 username, sizeof(username), NULL, NULL);
 
 	    StringCchLengthW(pInfo->Domain, MAX_DOMAIN_LENGTH, &szlen);
-	    WideCharToMultiByte(CP_UTF8, 0, pInfo->Domain, (int)szlen,
+	    WideCharToMultiByte(CP_UTF8, 0, pInfo->Domain, szlen,
 				 domain, sizeof(domain), NULL, NULL);
 
 	    GetDomainLogonOptions(NULL, username, domain, &opt);
@@ -1411,11 +1389,11 @@ VOID AFS_Logon_Event( PWLX_NOTIFICATION_INFO pInfo )
 	DebugEvent0("AFS_Logon_Event - pInfo UserName and Domain");
 
         StringCchLengthW(pInfo->UserName, MAX_USERNAME_LENGTH, &szlen);
-        WideCharToMultiByte(CP_UTF8, 0, pInfo->UserName, (int)szlen,
+        WideCharToMultiByte(CP_UTF8, 0, pInfo->UserName, szlen,
                             username, sizeof(username), NULL, NULL);
 
         StringCchLengthW(pInfo->Domain, MAX_DOMAIN_LENGTH, &szlen);
-        WideCharToMultiByte(CP_UTF8, 0, pInfo->Domain, (int)szlen,
+        WideCharToMultiByte(CP_UTF8, 0, pInfo->Domain, szlen,
                             domain, sizeof(domain), NULL, NULL);
 
 	DebugEvent0("AFS_Logon_Event - Calling GetDomainLogonOptions");
