@@ -1387,15 +1387,41 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
                                     AFSDeleteDirEntry( pCurrentObject,
                                                        pCurrentDirEntry);
 
-                                    if( pCurrentChildObject->ObjectReferenceCount <= 0)
+                                    if( pCurrentChildObject->ObjectReferenceCount <= 0 &&
+                                        pCurrentChildObject->Fcb != NULL &&
+                                        pCurrentChildObject->FileType == AFS_FILE_TYPE_FILE)
+                                    {
+
+                                        //
+                                        // We must not hold pVolumeCB->ObjectInfoTree.TreeLock exclusive
+                                        // across an AFSCleanupFcb call since it can deadlock with an
+                                        // invalidation call from the service.
+                                        //
+
+                                        AFSReleaseResource( pVolumeCB->ObjectInfoTree.TreeLock);
+
+                                        //
+                                        // Dropping the TreeLock permits the
+                                        // pCurrentObject->ObjectReferenceCount to change
+                                        //
+
+                                        AFSCleanupFcb( pCurrentChildObject->Fcb,
+                                                       TRUE);
+
+                                        AFSAcquireExcl( pVolumeCB->ObjectInfoTree.TreeLock,
+                                                        TRUE);
+                                    }
+
+                                    if( pCurrentChildObject->ObjectReferenceCount <= 0 &&
+                                        ( pCurrentChildObject->Fcb == NULL ||
+                                          pCurrentChildObject->Fcb->OpenReferenceCount == 0 &&
+                                          pCurrentChildObject->Fcb->Specific.File.ExtentCount == 0))
                                     {
 
                                         if( pCurrentChildObject->Fcb != NULL)
                                         {
-
-                                            pFcb = (AFSFcb *) InterlockedCompareExchangePointer( (PVOID *)&pCurrentChildObject->Fcb, NULL, (PVOID)pCurrentChildObject->Fcb);
-
-                                            lFileType = pCurrentChildObject->FileType;
+                                        
+                                            AFSRemoveFcb( &pCurrentChildObject->Fcb);
                                         }
 
                                         if( pCurrentChildObject->FileType == AFS_FILE_TYPE_DIRECTORY &&
@@ -1427,33 +1453,6 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
 
                                     pCurrentDirEntry = pNextDirEntry;
 
-                                    if ( pFcb != NULL)
-                                    {
-
-                                        if( lFileType == AFS_FILE_TYPE_FILE)
-                                        {
-                                            //
-                                            // We must not hold pVolumeCB->ObjectInfoTree.TreeLock exclusive
-                                            // across an AFSCleanupFcb call since it can deadlock with an
-                                            // invalidation call from the service.
-                                            //
-
-                                            AFSReleaseResource( pVolumeCB->ObjectInfoTree.TreeLock);
-
-                                            //
-                                            // Dropping the TreeLock permits the
-                                            // pCurrentObject->ObjectReferenceCount to change
-                                            //
-
-                                            AFSCleanupFcb( pFcb,
-                                                           TRUE);
-
-                                            AFSAcquireExcl( pVolumeCB->ObjectInfoTree.TreeLock,
-                                                            TRUE);
-                                        }
-
-                                        AFSRemoveFcb( &pFcb);
-                                    }
                                 }
 
                                 pCurrentObject->Specific.Directory.DirectoryNodeListHead = NULL;
@@ -1536,71 +1535,50 @@ AFSPrimaryVolumeWorkerThread( IN PVOID Context)
                     else if( pCurrentObject->FileType == AFS_FILE_TYPE_FILE)
                     {
 
-                        if( BooleanFlagOn( pCurrentObject->Flags, AFS_OBJECT_FLAGS_DELETED) &&
-                            pCurrentObject->ObjectReferenceCount <= 0 &&
-                            ( pCurrentObject->Fcb == NULL ||
-                              pCurrentObject->Fcb->OpenReferenceCount == 0))
+                        AFSReleaseResource( pVolumeCB->ObjectInfoTree.TreeLock);
+
+                        if( pCurrentObject->Fcb != NULL)
                         {
 
-                            pFcb = (AFSFcb *) InterlockedCompareExchangePointer( (PVOID *)&pCurrentObject->Fcb, NULL, (PVOID)pCurrentObject->Fcb);
-
-                            if( pFcb != NULL)
-                            {
-
-                                AFSReleaseResource( pVolumeCB->ObjectInfoTree.TreeLock);
-
-                                //
-                                // Dropping the TreeLock permits the
-                                // pCurrentObject->ObjectReferenceCount to change
-                                //
-
-                                AFSCleanupFcb( pFcb,
-                                               TRUE);
-
-                                AFSAcquireExcl( pVolumeCB->ObjectInfoTree.TreeLock,
-                                                TRUE);
-
-                                AFSRemoveFcb( &pFcb);
-                            }
-
-                            AFSReleaseResource( pVolumeCB->ObjectInfoTree.TreeLock);
-
-                            if( AFSAcquireExcl( pVolumeCB->ObjectInfoTree.TreeLock,
-                                                FALSE))
-                            {
-
-                                AFSDeleteObjectInfo( pCurrentObject);
-
-                                AFSConvertToShared( pVolumeCB->ObjectInfoTree.TreeLock);
-
-                                pCurrentObject = pNextObject;
-
-                                continue;
-                            }
-                            else
-                            {
-
-                                bReleaseVolumeLock = FALSE;
-
-                                break;
-                            }
-                        }
-                        else if( pCurrentObject->Fcb != NULL)
-                        {
-
-                            AFSReleaseResource( pVolumeCB->ObjectInfoTree.TreeLock);
-
                             //
                             // Dropping the TreeLock permits the
                             // pCurrentObject->ObjectReferenceCount to change
                             //
 
                             AFSCleanupFcb( pCurrentObject->Fcb,
-                                           FALSE);
-
-                            AFSAcquireShared( pVolumeCB->ObjectInfoTree.TreeLock,
-                                              TRUE);
+                                           TRUE);
                         }
+
+                        if( !AFSAcquireExcl( pVolumeCB->ObjectInfoTree.TreeLock,
+                                             FALSE))
+                        {
+
+                            bReleaseVolumeLock = FALSE;
+
+                            break;
+                        }
+
+                        if( BooleanFlagOn( pCurrentObject->Flags, AFS_OBJECT_FLAGS_DELETED) &&
+                            pCurrentObject->ObjectReferenceCount <= 0 &&
+                            ( pCurrentObject->Fcb == NULL ||
+                              pCurrentObject->Fcb->OpenReferenceCount == 0 &&
+                              pCurrentObject->Fcb->Specific.File.ExtentCount == 0))
+                        {
+
+                            if( pCurrentObject->Fcb != NULL)
+                            {
+
+                                AFSRemoveFcb( &pCurrentObject->Fcb);
+                            }
+
+                            AFSDeleteObjectInfo( pCurrentObject);
+                        }
+
+                        AFSConvertToShared( pVolumeCB->ObjectInfoTree.TreeLock);
+
+                        pCurrentObject = pNextObject;
+
+                        continue;
                     }
 
                     pCurrentObject = pNextObject;
@@ -2183,9 +2161,23 @@ AFSQueueFlushExtents( IN AFSFcb *Fcb,
 
         pWorkItem->RequestType = AFS_WORK_FLUSH_FCB;
 
-        RtlCopyMemory( &pWorkItem->AuthGroup,
-                       AuthGroup,
-                       sizeof( GUID));
+        if ( AuthGroup == NULL)
+        {
+
+            RtlZeroMemory( &pWorkItem->AuthGroup,
+                           sizeof( GUID));
+
+            ntStatus = AFSRetrieveValidAuthGroup( Fcb,
+                                                  NULL,
+                                                  TRUE,
+                                                  &pWorkItem->AuthGroup);
+        }
+        else
+        {
+            RtlCopyMemory( &pWorkItem->AuthGroup,
+                           AuthGroup,
+                           sizeof( GUID));
+        }
 
         pWorkItem->Specific.Fcb.Fcb = Fcb;
 
@@ -2249,12 +2241,14 @@ try_exit:
                           "AFSQueueFlushExtents Failed to queue request Status %08lX\n", ntStatus);
         }
     }
-    __except( AFSExceptionFilter( GetExceptionCode(), GetExceptionInformation()) )
+    __except( AFSExceptionFilter( __FUNCTION__, GetExceptionCode(), GetExceptionInformation()) )
     {
 
         AFSDbgLogMsg( 0,
                       0,
                       "EXCEPTION - AFSQueueFlushExtents\n");
+
+        AFSDumpTraceFilesFnc();
     }
 
     return ntStatus;
@@ -2333,12 +2327,14 @@ try_exit:
                           "AFSQueueAsyncRead Failed to queue request Status %08lX\n", ntStatus);
         }
     }
-    __except( AFSExceptionFilter( GetExceptionCode(), GetExceptionInformation()) )
+    __except( AFSExceptionFilter( __FUNCTION__, GetExceptionCode(), GetExceptionInformation()) )
     {
 
         AFSDbgLogMsg( 0,
                       0,
                       "EXCEPTION - AFSQueueAsyncRead\n");
+
+        AFSDumpTraceFilesFnc();
     }
 
     return ntStatus;
@@ -2417,12 +2413,14 @@ try_exit:
                           "AFSQueueAsyncWrite Failed to queue request Status %08lX\n", ntStatus);
         }
     }
-    __except( AFSExceptionFilter( GetExceptionCode(), GetExceptionInformation()) )
+    __except( AFSExceptionFilter( __FUNCTION__, GetExceptionCode(), GetExceptionInformation()) )
     {
 
         AFSDbgLogMsg( 0,
                       0,
                       "EXCEPTION - AFSQueueAsyncWrite\n");
+
+        AFSDumpTraceFilesFnc();
     }
 
     return ntStatus;
@@ -2487,12 +2485,14 @@ try_exit:
                           ntStatus);
         }
     }
-    __except( AFSExceptionFilter( GetExceptionCode(), GetExceptionInformation()) )
+    __except( AFSExceptionFilter( __FUNCTION__, GetExceptionCode(), GetExceptionInformation()) )
     {
 
         AFSDbgLogMsg( 0,
                       0,
                       "EXCEPTION - AFSQueueGlobalRootEnumeration\n");
+
+        AFSDumpTraceFilesFnc();
     }
 
     return ntStatus;
@@ -2591,12 +2591,14 @@ try_exit:
             }
         }
     }
-    __except( AFSExceptionFilter( GetExceptionCode(), GetExceptionInformation()) )
+    __except( AFSExceptionFilter( __FUNCTION__, GetExceptionCode(), GetExceptionInformation()) )
     {
 
         AFSDbgLogMsg( 0,
                       0,
                       "EXCEPTION - AFSQueueStartIos\n");
+
+        AFSDumpTraceFilesFnc();
     }
 
     return ntStatus;
@@ -2665,12 +2667,14 @@ try_exit:
                           ntStatus);
         }
     }
-    __except( AFSExceptionFilter( GetExceptionCode(), GetExceptionInformation()) )
+    __except( AFSExceptionFilter( __FUNCTION__, GetExceptionCode(), GetExceptionInformation()) )
     {
 
         AFSDbgLogMsg( 0,
                       0,
                       "EXCEPTION - AFSQueueInvalidateObject\n");
+
+        AFSDumpTraceFilesFnc();
     }
 
     return ntStatus;
