@@ -1,14 +1,43 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
  */
+/* AFSIFS portions copyright (c) 2005
+ * the regents of the university of michigan
+ * all rights reserved
+ *
+ * permission is granted to use, copy, create derivative works and
+ * redistribute this software and such derivative works for any purpose,
+ * so long as no fee is charged, and so long as the copyright notice
+ * above, this grant of permission, and the disclaimer below appear
+ * in all copies made; and so long as the name of the university of
+ * michigan is not used in any advertising or publicity pertaining
+ * to the use or distribution of this software without specific, written
+ * prior authorization.
+ *
+ * this software is provided as is, without representation from the
+ * university of michigan as to its fitness for any purpose, and without
+ * warranty by the university of michigan of any kind, either express
+ * or implied, including without limitation the implied warranties of
+ * merchantability and fitness for a particular purpose.  the regents
+ * of the university of michigan shall not be liable for nay damages,
+ * including special, indirect, incidental, or consequential damages,
+ * with respect to ant claim arising out of or in connection with the
+ * use of the software, even if it has been or is hereafter advised
+ * of the possibility of such damages.
+ */
+
+#include <winsock2.h>
+#include <ws2tcpip.h>
 
 extern "C" {
+#include <afsconfig.h>
 #include <afs/param.h>
+#include <roken.h>
 #include <afs/stds.h>
 #include <rx/rxkad.h>
 #include <afs/fs_utils.h>
@@ -52,7 +81,7 @@ static TCHAR cszAUTOSUBMOUNT[] = TEXT("Auto");
 static TCHAR cszLANMANDEVICE[] = TEXT("\\Device\\LanmanRedirector\\");
 
 
-static BOOL 
+static BOOL
 WriteRegistryString(HKEY key, TCHAR * subkey, LPTSTR lhs, LPTSTR rhs)
 {
     HKEY hkSub = NULL;
@@ -74,7 +103,7 @@ WriteRegistryString(HKEY key, TCHAR * subkey, LPTSTR lhs, LPTSTR rhs)
     return (status == ERROR_SUCCESS);
 }
 
-static BOOL 
+static BOOL
 WriteExpandedRegistryString(HKEY key, TCHAR * subkey, LPTSTR lhs, LPTSTR rhs)
 {
     HKEY hkSub = NULL;
@@ -96,7 +125,7 @@ WriteExpandedRegistryString(HKEY key, TCHAR * subkey, LPTSTR lhs, LPTSTR rhs)
     return (status == ERROR_SUCCESS);
 }
 
-static BOOL 
+static BOOL
 ReadRegistryString(HKEY key, TCHAR * subkey, LPTSTR lhs, LPTSTR rhs, DWORD * size)
 {
     HKEY hkSub = NULL;
@@ -130,7 +159,7 @@ ReadRegistryString(HKEY key, TCHAR * subkey, LPTSTR lhs, LPTSTR rhs, DWORD * siz
     return (status == ERROR_SUCCESS);
 }
 
-static BOOL 
+static BOOL
 DeleteRegistryString(HKEY key, TCHAR * subkey, LPTSTR lhs)
 {
     HKEY hkSub = NULL;
@@ -169,6 +198,7 @@ static void FreeStringMemory (LPTSTR pszString)
    Free (pszString);
 }
 
+#if 0
 static int lstrncmpi (LPCTSTR pszA, LPCTSTR pszB, size_t cch)
 {
    if (!pszA || !pszB)
@@ -190,7 +220,7 @@ static int lstrncmpi (LPCTSTR pszA, LPCTSTR pszB, size_t cch)
 
    return 0;  // no differences before told to stop comparing, so A==B
 }
-
+#endif
 
 /*
  * REALLOC ____________________________________________________________________
@@ -314,18 +344,27 @@ BOOL SubmountToPath (PDRIVEMAPLIST pList, LPTSTR pszPath, LPTSTR pszSubmount, BO
 
    // Otherwise, look up our list of submounts.
    //
-   for (size_t ii = 0; ii < pList->cSubmounts; ++ii)
-      {
-      if (!lstrcmpi (pList->aSubmounts[ii].szSubmount, pszSubmount))
-         {
-         if (fMarkInUse)
-            pList->aSubmounts[ii].fInUse = TRUE;
-         AdjustAfsPath (pszPath, pList->aSubmounts[ii].szMapping, TRUE, TRUE);
-         return TRUE;
-         }
-      }
+#ifdef AFSIFS
+    AdjustAfsPath (pszPath, pszSubmount, TRUE, TRUE);
+#endif
+    for (size_t ii = 0; ii < pList->cSubmounts; ++ii)
+    {
+        BOOL b;
+#ifndef AFSIFS
+        b = !lstrcmpi (pList->aSubmounts[ii].szSubmount, pszSubmount);
+#else
+        b = !lstrcmpi (pList->aSubmounts[ii].szMapping, pszPath);
+#endif
+        if (b)
+        {
+            if (fMarkInUse)
+                pList->aSubmounts[ii].fInUse = TRUE;
+            AdjustAfsPath (pszPath, pList->aSubmounts[ii].szMapping, TRUE, TRUE);
+            return TRUE;
+        }
+    }
 
-   return FALSE;
+    return FALSE;
 }
 
 
@@ -366,6 +405,7 @@ extern "C" {
 #define __CM_CONFIG_INTERFACES_ONLY__
 #include "../afsd/cm_config.h"
 
+#include "../afsd/cm_nls.h"
 #define __CM_IOCTL_INTERFACES_ONLY__
 #include "../afsd/cm_ioctl.h"
 
@@ -415,13 +455,13 @@ void QueryDriveMapList_ReadSubmounts (PDRIVEMAPLIST pList)
     {
         HKEY hkSubmounts;
 
-        RegCreateKeyEx( HKEY_LOCAL_MACHINE, 
+        RegCreateKeyEx( HKEY_LOCAL_MACHINE,
                         cszSECTION_SUBMOUNTS,
-                        0, 
-                        "AFS", 
+                        0,
+                        "AFS",
                         REG_OPTION_NON_VOLATILE,
                         KEY_READ|KEY_QUERY_VALUE,
-                        NULL, 
+                        NULL,
                         &hkSubmounts,
                         NULL );
 
@@ -488,11 +528,11 @@ void QueryDriveMapList_ReadMappings (PDRIVEMAPLIST pList)
     HKEY hkMappings;
     RegCreateKeyEx( HKEY_CURRENT_USER,
                     cszSECTION_MAPPINGS,
-                    0, 
-                    "AFS", 
+                    0,
+                    "AFS",
                     REG_OPTION_NON_VOLATILE,
                     KEY_READ|KEY_QUERY_VALUE,
-                    NULL, 
+                    NULL,
                     &hkMappings,
                     NULL );
 
@@ -587,13 +627,13 @@ void QueryDriveMapList_WriteMappings (PDRIVEMAPLIST pList)
 void WriteDriveMappings (PDRIVEMAPLIST pList)
 {
     HKEY hkMappings;
-    RegCreateKeyEx( HKEY_CURRENT_USER, 
+    RegCreateKeyEx( HKEY_CURRENT_USER,
                     cszSECTION_MAPPINGS,
-                    0, 
-                    "AFS", 
+                    0,
+                    "AFS",
                     REG_OPTION_NON_VOLATILE,
                     KEY_READ|KEY_QUERY_VALUE|KEY_WRITE,
-                    NULL, 
+                    NULL,
                     &hkMappings,
                     NULL );
 
@@ -658,7 +698,7 @@ BOOL DriveIsGlobalAfsDrive(TCHAR chDrive)
    BOOL bIsGlobal = (RegQueryValueEx (hKey, szValueName, NULL, NULL, (PBYTE)szValue, &dwSize) == ERROR_SUCCESS);
 
    RegCloseKey (hKey);
-   
+
    return bIsGlobal;
 }
 
@@ -691,13 +731,13 @@ void QueryDriveMapList_FindNetworkDrives (PDRIVEMAPLIST pList, BOOL *pfFoundNew)
             continue;
 
          // Okay, we know that drive {chDrive} is mapped to afs path {szAfsPath}.
-         // If this drive is a global afs drive, then reject it.  Otherwise, look 
-         // at pList->aDriveMap, to see if this drive mapping is already in our 
+         // If this drive is a global afs drive, then reject it.  Otherwise, look
+         // at pList->aDriveMap, to see if this drive mapping is already in our
          // list. If not, add it and set pfFoundNew.
          //
          if (DriveIsGlobalAfsDrive(chDrive))
             continue;
-         
+
          size_t iDrive = chDrive - chDRIVE_A;
          if (lstrcmpi (pList->aDriveMap[ iDrive ].szMapping, szAfsPath))
             {
@@ -748,8 +788,11 @@ void FreeDriveMapList (PDRIVEMAPLIST pList)
 }
 
 
+
 BOOL PathToSubmount (LPTSTR pszSubmount, LPTSTR pszMapping, LPTSTR pszSubmountReq, ULONG *pStatus)
 {
+   // pszSubmount is MAX_PATH in length
+
    if (pszSubmountReq && !IsValidSubmountName (pszSubmountReq))
       pszSubmountReq = NULL;
 
@@ -786,6 +829,8 @@ BOOL PathToSubmount (LPTSTR pszSubmount, LPTSTR pszMapping, LPTSTR pszSubmountRe
    if (status)
       return FALSE;
 
+
+   OutData[min(PIOCTL_MAXSIZE, MAX_PATH) - 1] = '\0';
    lstrcpy (pszSubmount, (LPCTSTR)OutData);
    return (pszSubmount[0] != TEXT('\0')) ? TRUE : FALSE;
 }
@@ -816,13 +861,18 @@ BOOL ActivateDriveMap (TCHAR chDrive, LPTSTR pszMapping, LPTSTR pszSubmountReq,
       }
 
    // We now have a submount name and drive letter--map the network drive.
-   DWORD rc=MountDOSDrive(chDrive,szSubmount,fPersistent,NULL);
-   if (rc == NO_ERROR)
-      return TRUE;
+    DWORD rc;
+#ifndef AFSIFS
+    rc=MountDOSDrive(chDrive,szSubmount,fPersistent,NULL);
+#else
+    rc=MountDOSDrive(chDrive,/*szSubmount*/pszMapping,fPersistent,NULL);
+#endif
+    if (rc == NO_ERROR)
+        return TRUE;
 
-   if (pdwStatus)
-      *pdwStatus = rc;
-   return FALSE;
+    if (pdwStatus)
+        *pdwStatus = rc;
+    return FALSE;
 }
 
 
@@ -906,7 +956,7 @@ BOOL GetDriveSubmount (TCHAR chDrive, LPTSTR pszSubmountNow)
            return FALSE;
 
     LPTSTR pszSubmount = szMapping;
-    
+
 	TCHAR szNetBiosName[32];
     memset(szNetBiosName, '\0', sizeof(szNetBiosName));
     GetClientNetbiosName(szNetBiosName);
@@ -932,16 +982,33 @@ BOOL GetDriveSubmount (TCHAR chDrive, LPTSTR pszSubmountNow)
       //           <AuthID>: Authentication ID, 16 char hex.
       //           <netbiosname>: Netbios name of server
       //
-      if (_tcsnicmp(szMapping, cszLANMANDEVICE, _tcslen(cszLANMANDEVICE)))
+      BOOL b;
+#ifndef AFSIFS
+      b = _tcsnicmp(szMapping, cszLANMANDEVICE, _tcslen(cszLANMANDEVICE));
+#else
+      const TCHAR ker_sub_path[] = "\\Device\\afsrdr\\";
+      b = _tcsnicmp(szMapping, ker_sub_path, _tcslen(ker_sub_path));
+#endif
+       if (b)
          return FALSE;
+#ifndef AFSIFS
       pszSubmount = &szMapping[ _tcslen(cszLANMANDEVICE) ];
+#else
+      pszSubmount = &szMapping[ _tcslen(ker_sub_path) ];
+#endif
 
+#ifdef AFSIFS
+       if (*(pszSubmount) < '0' ||
+            *(pszSubmount) > '9')
+           return FALSE;
+       ++pszSubmount;
+#else
       if (IsWindows2000())
-	  {
+      {
           if (*(pszSubmount) != TEXT(';'))
              return FALSE;
-	  } else 
-		--pszSubmount;
+      } else
+          --pszSubmount;
 
       if (toupper(*(++pszSubmount)) != chDrive)
          return FALSE;
@@ -971,6 +1038,7 @@ BOOL GetDriveSubmount (TCHAR chDrive, LPTSTR pszSubmountNow)
          return FALSE;
 
        pszSubmount += _tcslen(szNetBiosName);
+#endif
       }
    else // (!IsWindowsNT())
       {
@@ -992,7 +1060,12 @@ BOOL GetDriveSubmount (TCHAR chDrive, LPTSTR pszSubmountNow)
    if (!pszSubmount || !*pszSubmount)
       return FALSE;
 
+#ifndef AFSIFS
    lstrcpy (pszSubmountNow, pszSubmount);
+#else
+   lstrcpy (pszSubmountNow, "\\afs");
+   lstrcat (pszSubmountNow, pszSubmount);
+#endif
    return TRUE;
 }
 
@@ -1005,7 +1078,7 @@ BOOL fUserName=FALSE;
 void SetBitLogonOption(BOOL set,DWORD value)
 {
 
-    RWLogonOption(FALSE,((set)?value | RWLogonOption(TRUE,0):RWLogonOption(TRUE,0) & ~value) );	
+    RWLogonOption(FALSE,((set)?value | RWLogonOption(TRUE,0):RWLogonOption(TRUE,0) & ~value) );
 }
 
 DWORD RWLogonOption(BOOL read,DWORD value)
@@ -1016,7 +1089,7 @@ DWORD RWLogonOption(BOOL read,DWORD value)
     DWORD dwDisp;
     DWORD LSPtype, LSPsize;
     DWORD rval;
-   
+
     if (read)
     {
         rval=0;
@@ -1035,7 +1108,7 @@ DWORD RWLogonOption(BOOL read,DWORD value)
             RegCloseKey (hk);
         }
         return TRUE;
-    }    
+    }
 }
 
 void MapShareName(char *pszCmdLineA)
@@ -1076,8 +1149,6 @@ BOOL TestAndDoMapShare(DWORD dwState)
 	return TRUE;
     }
     dwOldState=SERVICE_RUNNING;
-    if (RWLogonOption(TRUE,LOGON_OPTION_HIGHSECURITY))
-	return (DoMapShare() && GlobalMountDrive());
     return GlobalMountDrive();
 }
 
@@ -1102,12 +1173,10 @@ BOOL IsServiceActive()
 
 void TestAndDoUnMapShare()
 {
-    if (!RWLogonOption(TRUE,LOGON_OPTION_HIGHSECURITY))
-	return;
-    DoUnMapShare(FALSE);	
+    return;
 }
 
-void DoUnMapShare(BOOL drivemap)	//disconnect drivemap 
+void DoUnMapShare(BOOL drivemap)	//disconnect drivemap
 {
     TCHAR szMachine[MAX_PATH],szPath[MAX_PATH];
     DWORD rc=28;
@@ -1140,7 +1209,7 @@ void DoUnMapShare(BOOL drivemap)	//disconnect drivemap
 			if (drivemap) {
 			    DisMountDOSDrive(*lpnrLocal[i].lpLocalName);
                             DEBUG_EVENT1("AFS DriveUnMap","UnMap-Local=%x",res);
-                        }		
+                        }
 		    } else {
 			DisMountDOSDriveFull(lpnrLocal[i].lpRemoteName);
                         DEBUG_EVENT1("AFS DriveUnMap","UnMap-Remote=%x",res);
@@ -1188,15 +1257,15 @@ BOOL DoMapShareChange(BOOL removeUnknown)
 		if (strstr(_strlwr(lpnrLocal[i].lpRemoteName),szPath)==NULL)
 		    continue;	//only look at real afs mappings
 		CHAR * pSubmount=strrchr(lpnrLocal[i].lpRemoteName,'\\')+1;
-		if (lstrcmpi(pSubmount,"all")==0) 
+		if (lstrcmpi(pSubmount,"all")==0)
 		    continue;				// do not remove 'all'
 		for (DWORD j=0;j<List.cSubmounts;j++)
 		{
 		    if ((List.aSubmounts[j].szSubmount[0]) &&
 			 (lstrcmpi(List.aSubmounts[j].szSubmount,pSubmount)==0)
-			 ) 
+			 )
 		    {
-			List.aSubmounts[j].fInUse=TRUE; 
+			List.aSubmounts[j].fInUse=TRUE;
 			goto nextname;
 		    }
 		}
@@ -1214,16 +1283,7 @@ BOOL DoMapShareChange(BOOL removeUnknown)
     DWORD cbUser=MAXRANDOMNAMELEN-1;
     CHAR szUser[MAXRANDOMNAMELEN];
     CHAR * pUser = NULL;
-    if (WNetGetUser(szPath,(LPSTR)szUser,&cbUser)!=NO_ERROR) {
-        if (RWLogonOption(TRUE,LOGON_OPTION_HIGHSECURITY)) {
-            if (!pUserName[0]) {
-                GenRandomName(szUser,MAXRANDOMNAMELEN-1);
-                pUser = szUser;
-            } else {
-                pUser = pUserName;
-            }
-        }
-    } else {
+    if (WNetGetUser(szPath,(LPSTR)szUser,&cbUser)==NO_ERROR) {
 	if ((pUser=strchr(szUser,'\\'))!=NULL)
             pUser++;
     }
@@ -1259,16 +1319,7 @@ BOOL DoMapShare()
     DWORD cbUser=MAXRANDOMNAMELEN-1;
     CHAR szUser[MAXRANDOMNAMELEN];
     CHAR * pUser = NULL;
-    if (WNetGetUser(szPath,(LPSTR)szUser,&cbUser)!=NO_ERROR) {
-        if (RWLogonOption(TRUE,LOGON_OPTION_HIGHSECURITY)) {
-            if (!pUserName[0]) {
-                GenRandomName(szUser,MAXRANDOMNAMELEN-1);
-                pUser = szUser;
-            } else {
-                pUser = pUserName;
-            }
-        }
-    } else {
+    if (WNetGetUser(szPath,(LPSTR)szUser,&cbUser)==NO_ERROR) {
 	if ((pUser=strchr(szUser,'\\'))!=NULL)
             pUser++;
     }
@@ -1303,8 +1354,6 @@ BOOL DoMapShare()
                 continue;
 
             BOOL fPersistent = List.aDriveMap[chDrive-chDRIVE_A].fPersistent;
-            if (RWLogonOption(TRUE,LOGON_OPTION_HIGHSECURITY))
-                fPersistent = FALSE;
 	    DWORD res=MountDOSDrive(chDrive
 				     ,szSubmount
 				     ,fPersistent,pUser);
@@ -1330,16 +1379,16 @@ BOOL GlobalMountDrive()
     if (!GetComputerName(cm_HostName, &dwType))
         return TRUE;
     sprintf(szKeyName, "%s\\GlobalAutoMapper", AFSREG_CLT_SVC_PARAM_SUBKEY);
-    
+
     dwResult = RegOpenKeyEx(HKEY_LOCAL_MACHINE, szKeyName, 0, KEY_QUERY_VALUE,
 			    &hKey);
     if (dwResult != ERROR_SUCCESS)
 	return TRUE;
-    
+
     while (1) {
         dwDriveSize = sizeof(szDriveToMapTo);
         dwSubMountSize = sizeof(szSubMount);
-        dwResult = RegEnumValue(hKey, dwIndex++, szDriveToMapTo, &dwDriveSize, 
+        dwResult = RegEnumValue(hKey, dwIndex++, szDriveToMapTo, &dwDriveSize,
 				0, &dwType, szSubMount, &dwSubMountSize);
         if (dwResult != ERROR_MORE_DATA) {
 	    if (dwResult != ERROR_SUCCESS) {
@@ -1358,9 +1407,44 @@ BOOL GlobalMountDrive()
 
 DWORD MountDOSDrive(char chDrive,const char *szSubmount,BOOL bPersistent,const char * pUsername)
 {
+#ifdef AFSIFS
+    DWORD err;
+    BOOL succ;
+    TCHAR szTokens[MAX_PATH], *tok;
+#endif /* AFSIFS */
     TCHAR szPath[MAX_PATH];
     TCHAR szClient[MAX_PATH];
     TCHAR szDrive[3] = TEXT("?:");
+
+#ifdef AFSIFS
+    int pathCount, currPos, lastPos, x;
+
+    pathCount = 0;
+
+    pathCount = 0;
+    strcpy(szTokens, szSubmount);
+    tok = strtok(szTokens, "/\\");
+    strcpy(szPath, "");
+    while (tok)
+    {
+        if (pathCount || stricmp(tok, "afs"))
+        {
+            strcat(szPath, "\\");
+            strcat(szPath, tok);
+            pathCount++;
+        }
+        tok = strtok(NULL, "/\\");
+    }
+
+    sprintf(szDrive,"%c:",chDrive);
+    strcpy(szTokens, szPath);
+    sprintf(szPath,"\\Device\\afsrdr\\%d%s",pathCount,szTokens);
+    //succ = DefineDosDevice(DDD_RAW_TARGET_PATH, "J:", "\\Device\\afsrdr\\2\\ericjw\\test");
+    succ = DefineDosDevice(DDD_RAW_TARGET_PATH, szDrive, szPath);
+    err = GetLastError();
+
+    return succ ? NO_ERROR : ERROR_DEVICE_IN_USE;
+#else
     sprintf(szDrive,"%c:",chDrive);
     GetClientNetbiosName (szClient);
     sprintf(szPath,"\\\\%s\\%s",szClient,szSubmount);
@@ -1375,11 +1459,19 @@ DWORD MountDOSDrive(char chDrive,const char *szSubmount,BOOL bPersistent,const c
                   (bPersistent)?"Persistant" : "NonPresistant",
                   szDrive,szPath,pUsername?pUsername:"NULL",res);
     return res;
+#endif
 }
 
 DWORD DisMountDOSDriveFull(const char *szPath,BOOL bForce)
 {
+#ifndef AFSIFS
     DWORD res=WNetCancelConnection(szPath,bForce);
+#else
+    DWORD res;
+    res = ERROR_DEVICE_IN_USE;
+    // must handle drive letters and afs paths
+    // DDD_REMOVE_DEFINITION
+#endif
     DEBUG_EVENT3("AFS DriveMap","%sDismount Remote[%s]=%x",
                   bForce ? "Forced " : "",szPath,res);
     return (res==ERROR_NOT_CONNECTED)?NO_ERROR:res;
@@ -1398,6 +1490,14 @@ DWORD DisMountDOSDrive(const char *pSubmount,BOOL bForce)
 DWORD DisMountDOSDrive(const char chDrive,BOOL bForce)
 {
     TCHAR szPath[MAX_PATH];
+#ifdef AFSIFS
+    DWORD succ;
+#endif
     sprintf(szPath,"%c:",chDrive);
+#ifdef AFSIFS
+    succ = DefineDosDevice(DDD_REMOVE_DEFINITION, szPath, NULL);
+    return (!succ) ? GetLastError() : 0;
+#else
     return DisMountDOSDriveFull(szPath,bForce);
+#endif
 }
