@@ -28,16 +28,16 @@
 
 
 #if defined(AFS_SUN5_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
-int afs_ustrategy(register struct buf *abp, afs_ucred_t *credp)
+int afs_ustrategy(struct buf *abp, afs_ucred_t *credp)
 #else
-int afs_ustrategy(register struct buf *abp)
+int afs_ustrategy(struct buf *abp)
 #endif
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct uio tuio;
     struct iovec tiovec[1];
-    register struct vcache *tvc = VTOAFS(abp->b_vp);
-    register afs_int32 len = abp->b_bcount;
+    struct vcache *tvc = VTOAFS(abp->b_vp);
+    afs_int32 len = abp->b_bcount;
 #ifdef	AFS_AIX41_ENV
     struct ucred *credp;
 #elif defined(UKERNEL)
@@ -61,7 +61,7 @@ int afs_ustrategy(register struct buf *abp)
     ReleaseReadLock(&tvc->lock);
     osi_Assert(credp);
 #endif
-#ifdef AFS_FBSD50_ENV
+#ifdef AFS_FBSD_ENV
     if (abp->b_iocmd == BIO_READ) {
 #else
     if ((abp->b_flags & B_READ) == B_READ) {
@@ -86,12 +86,18 @@ int afs_ustrategy(register struct buf *abp)
 #else
 	tuio.afsio_offset = DEV_BSIZE * abp->b_blkno;
 #endif
+#if defined(AFS_NBSD40_ENV)
+	UIO_SETUP_SYSSPACE(&tuio);
+#else
 	tuio.afsio_seg = AFS_UIOSYS;
+#endif
 #ifdef AFS_UIOFMODE
 	tuio.afsio_fmode = 0;
 #endif
 	tuio.afsio_resid = abp->b_bcount;
-#if defined(AFS_XBSD_ENV)
+#if defined(AFS_NBSD40_ENV)
+	tiovec[0].iov_base = abp->b_data;
+#elif defined(AFS_XBSD_ENV)
 	tiovec[0].iov_base = abp->b_saveaddr;
 #else
 	tiovec[0].iov_base = abp->b_un.b_addr;
@@ -106,7 +112,10 @@ int afs_ustrategy(register struct buf *abp)
 #endif
 	if (code == 0) {
 	    if (tuio.afsio_resid > 0)
-#if defined(AFS_XBSD_ENV)
+#if defined(AFS_NBSD40_ENV)
+		memset((char *)abp->b_data + (uintptr_t)abp->b_bcount - tuio.afsio_resid, 0,
+		       tuio.afsio_resid);
+#elif defined(AFS_XBSD_ENV)
 		memset(abp->b_saveaddr + abp->b_bcount - tuio.afsio_resid, 0,
 		       tuio.afsio_resid);
 #else
@@ -152,7 +161,11 @@ int afs_ustrategy(register struct buf *abp)
 #else
 	tuio.afsio_offset = DEV_BSIZE * abp->b_blkno;
 #endif
+#if defined(AFS_NBSD40_ENV)
+	UIO_SETUP_SYSSPACE(&tuio);
+#else
 	tuio.afsio_seg = AFS_UIOSYS;
+#endif
 #ifdef AFS_UIOFMODE
 	tuio.afsio_fmode = 0;
 #endif
@@ -164,7 +177,9 @@ int afs_ustrategy(register struct buf *abp)
 	len = MIN(len, tvc->f.m.Length - dbtob(abp->b_blkno));
 #endif
 	tuio.afsio_resid = len;
-#if defined(AFS_XBSD_ENV)
+#if defined(AFS_NBSD40_ENV)
+	tiovec[0].iov_base = abp->b_data;
+#elif defined(AFS_XBSD_ENV)
 	tiovec[0].iov_base = abp->b_saveaddr;
 #else
 	tiovec[0].iov_base = abp->b_un.b_addr;
@@ -182,7 +197,7 @@ int afs_ustrategy(register struct buf *abp)
 #if defined (AFS_XBSD_ENV)
     if (code) {
 	abp->b_error = code;
-#if !defined(AFS_FBSD50_ENV)
+#if !defined(AFS_FBSD_ENV) && !defined(AFS_NBSD50_ENV)
 	abp->b_flags |= B_ERROR;
 #endif
     }
@@ -192,8 +207,11 @@ int afs_ustrategy(register struct buf *abp)
     crfree(credp);
 #elif defined(AFS_FBSD60_ENV)
     (*abp->b_iodone)(abp);
-#elif defined(AFS_FBSD50_ENV)
+#elif defined(AFS_FBSD_ENV)
     biodone(&abp->b_io);
+#elif defined(AFS_NBSD40_ENV)
+    abp->b_resid = tuio.uio_resid;
+    biodone(abp);
 #elif defined(AFS_XBSD_ENV)
     biodone(abp);
 #elif !defined(AFS_SUN5_ENV)
