@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -10,23 +10,19 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <afs/procmgmt.h>
+#include <roken.h>
 
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <lwp.h>
-#include <time.h>
 #include <ctype.h>
-#include <stdio.h>
+
 #ifdef AFS_NT40_ENV
 #include <io.h>
 #endif
 
-#include <string.h>
-#include <stdlib.h>
-
+#include <lwp.h>
 #include <afs/ktime.h>
 #include <afs/afsutil.h>
-#include <afs/procmgmt.h>	/* signal(), kill(), wait(), etc. */
+
 #include "bnode.h"
 #include "bosprototypes.h"
 
@@ -37,6 +33,7 @@ static int cron_delete(struct bnode *bnode);
 static int cron_timeout(struct bnode *bnode);
 static int cron_getstat(struct bnode *bnode, afs_int32 *status);
 static int cron_setstat(struct bnode *bnode, afs_int32 status);
+static int cron_procstarted(struct bnode *bnode, struct bnode_proc *proc);
 static int cron_procexit(struct bnode *bnode, struct bnode_proc *proc);
 static int cron_getstring(struct bnode *bnode, char *abuffer, afs_int32 alen);
 static int cron_getparm(struct bnode *bnode, afs_int32, char *, afs_int32);
@@ -54,6 +51,7 @@ struct bnode_ops cronbnode_ops = {
     cron_getparm,
     cron_restartp,
     cron_hascore,
+    cron_procstarted,
 };
 
 struct cronbnode {
@@ -89,10 +87,10 @@ cron_hascore(struct bnode *abnode)
     Note that the computation of when we should run again is made in procexit
     and/or create procs.  This guy only schedules the sleep */
 int
-ScheduleCronBnode(register struct cronbnode *abnode)
+ScheduleCronBnode(struct cronbnode *abnode)
 {
-    register afs_int32 code;
-    register afs_int32 temp;
+    afs_int32 code;
+    afs_int32 temp;
     struct bnode_proc *tp;
 
     /* If this proc is shutdown, tell bproc() to no longer run this job */
@@ -144,7 +142,7 @@ cron_restartp(struct bnode *abnode)
 }
 
 static int
-cron_delete(struct bnode *bn) 
+cron_delete(struct bnode *bn)
 {
     struct cronbnode *abnode = (struct cronbnode *)bn;
     free(abnode->command);
@@ -187,8 +185,8 @@ static int
 cron_timeout(struct bnode *bn)
 {
     struct cronbnode *abnode = (struct cronbnode *)bn;
-    register afs_int32 temp;
-    register afs_int32 code;
+    afs_int32 temp;
+    afs_int32 code;
     struct bnode_proc *tp;
 
     if (!abnode->running) {
@@ -229,7 +227,7 @@ static int
 cron_getstat(struct bnode *bn, afs_int32 * astatus)
 {
     struct cronbnode *abnode = (struct cronbnode *)bn;
-    register afs_int32 temp;
+    afs_int32 temp;
     if (abnode->waitingForShutdown)
 	temp = BSTAT_SHUTTINGDOWN;
     else if (abnode->b.goal == 0)
@@ -275,6 +273,12 @@ cron_setstat(struct bnode *bn, afs_int32 astatus)
     return 0;
 }
 
+static int
+cron_procstarted(struct bnode *bn, struct bnode_proc *aproc)
+{
+    return 0;	/* no op */
+}
+
 static int
 cron_procexit(struct bnode *bn, struct bnode_proc *aproc)
 {
