@@ -10,6 +10,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
 
 #include <rx/xdr.h>
 #include <afs/kauth.h>
@@ -33,17 +34,7 @@ xdr_ka_CBS(XDR * x, struct ka_CBS *abbs)
 	xdr_afs_int32(x, &len);
 	if (len < 0 || len > MAXBS)
 	    return FALSE;
-	if (abbs->SeqBody != NULL) {
-	    if (len > abbs->SeqLen) {
-		/*
-		 * We've been given a preallocated buffer to decode into, but
-		 * we're decoding 'len' bytes, which is larger than the
-		 * provided buffer (only abbs->SeqLen bytes large). This won't
-		 * work.
-		 */
-		return FALSE;
-	    }
-	} else
+	if (!abbs->SeqBody)
 	    abbs->SeqBody = (char *)malloc(len);
 	abbs->SeqLen = len;
 	xdr_opaque(x, abbs->SeqBody, len);
@@ -70,26 +61,7 @@ xdr_ka_BBS(XDR * x, struct ka_BBS *abbs)
 	if (!xdr_afs_int32(x, &maxLen) || !xdr_afs_int32(x, &len) || (len < 0)
 	    || (len > MAXBS) || (len > maxLen))
 	    return FALSE;
-	if (abbs->SeqBody != NULL) {
-	    if (len > abbs->MaxSeqLen) {
-		/*
-		 * We've been given a preallocated buffer to decode into, but
-		 * we're decoding 'len' bytes, which is larger than the
-		 * provided buffer (only abbs->MaxSeqLen bytes large). This
-		 * won't work.
-		 */
-		return FALSE;
-	    }
-	    if (maxLen > abbs->MaxSeqLen) {
-		/*
-		 * Our preallocated buffer only has space for MaxSeqLen bytes.
-		 * Don't let the peer change 'abbs' so that it looks like we
-		 * have space for more bytes than that; that could cause us to
-		 * access memory beyond what we've actually allocated.
-		 */
-		return FALSE;
-	    }
-	} else
+	if (!abbs->SeqBody)
 	    abbs->SeqBody = (char *)malloc(maxLen);
 	abbs->MaxSeqLen = maxLen;
 	abbs->SeqLen = len;
